@@ -44,101 +44,6 @@
 #define DEBUG_FH stderr
 
 
-#if RAPTOR_VERSION < 20005
-
-static int
-rasqal_unicode_utf8_strlen(const unsigned char *string, size_t length)
-{
-  size_t unicode_length = 0;
-  
-  while(length > 0) {
-    int unichar_len;
-    unichar_len = raptor_unicode_utf8_string_get_char(string, length, NULL);
-    if(unichar_len < 0 || unichar_len > (int)length) {
-      unicode_length = -1;
-      break;
-    }
-    
-    string += unichar_len;
-    length -= unichar_len;
-
-    unicode_length++;
-  }
-
-  return (int)unicode_length;
-}
-
-
-/*
- * rasqal_unicode_utf8_substr:
- * @dest: destination string buffer to write to (or NULL)
- * @dest_length_p: location to store actual destination length (or NULL)
- * @src: source string
- * @src_length: source length in bytes
- * @startingLoc: starting location offset 0 for first Unicode character
- * @length: number of Unicode characters to copy at offset @startingLoc (or < 0)
- *
- * INTERNAL - Get a unicode (UTF-8) substring of an existing UTF-8 string
- *
- * If @dest is NULL, returns the number of bytes needed to write and
- * does no work.
- * 
- * Return value: number of bytes used in destination string or 0 on failure
- */
-static size_t
-rasqal_unicode_utf8_substr(unsigned char* dest, size_t* dest_length_p,
-                           const unsigned char* src, size_t src_length,
-                           int startingLoc, int length)
-{
-  size_t dest_length = 0; /* destination unicode characters count */
-  size_t dest_bytes = 0;  /* destination UTF-8 bytes count */
-  int dest_offset = 0; /* destination string unicode characters index */
-  unsigned char* p = dest;
-  
-  if(!src)
-    return 0;
-
-  while(src_length > 0) {
-    int unichar_len;
-
-    unichar_len = raptor_unicode_utf8_string_get_char(src, src_length, NULL);
-    if(unichar_len < 0 || unichar_len > (int)src_length)
-      break;
-
-    if(dest_offset >= startingLoc) {
-      if(p) {
-        /* copy 1 Unicode character to dest */
-        memcpy(p, src, unichar_len);
-        p += unichar_len;
-      }
-      dest_bytes += unichar_len;
-
-      dest_length++;
-      if(length >= 0 && (int)dest_length == length)
-        break;
-    }
-
-    src += unichar_len;
-    src_length -= unichar_len;
-
-    dest_offset++;
-  }
-
-  if(p)
-    *p = '\0';
-
-  if(dest_length_p)
-    *dest_length_p = dest_length;
-
-  return dest_bytes;
-}
-
-#define raptor_unicode_utf8_strlen rasqal_unicode_utf8_strlen
-#define raptor_unicode_utf8_substr rasqal_unicode_utf8_substr
-
-#endif
-
-
 /* 
  * rasqal_expression_evaluate_strlen:
  * @e: The expression to evaluate.
@@ -170,7 +75,7 @@ rasqal_expression_evaluate_strlen(rasqal_expression *e,
   if(!s)
     len = 0;
   else
-    len = raptor_unicode_utf8_strlen(s, strlen((const char*)s));
+    len = raptor_unicode_utf8_strlen(s, strlen(RASQAL_GOOD_CAST(const char*, s)));
   
 
   result = rasqal_new_numeric_literal_from_long(world, RASQAL_LITERAL_INTEGER,
@@ -255,7 +160,7 @@ rasqal_expression_evaluate_substr(rasqal_expression *e,
     goto failed;
 
   if(l1->language) {
-    len = strlen((const char*)l1->language);
+    len = strlen(RASQAL_GOOD_CAST(const char*, l1->language));
     new_lang = RASQAL_MALLOC(char*, len + 1);
     if(!new_lang)
       goto failed;
@@ -333,8 +238,8 @@ rasqal_expression_evaluate_set_case(rasqal_expression *e,
     
     for(i = 0; i < len; i++) {
       char c = s[i];
-      if(islower((int)c))
-        c = toupper((int)c);
+      if(islower(RASQAL_GOOD_CAST(int, c)))
+        c = toupper(RASQAL_GOOD_CAST(int, c));
       new_s[i] = c;
     }
   } else { /* RASQAL_EXPR_LCASE */
@@ -342,15 +247,15 @@ rasqal_expression_evaluate_set_case(rasqal_expression *e,
 
     for(i = 0; i < len; i++) {
       char c = s[i];
-      if(isupper((int)c))
-        c = tolower((int)c);
+      if(isupper(RASQAL_GOOD_CAST(int, c)))
+        c = tolower(RASQAL_GOOD_CAST(int, c));
       new_s[i] = c;
     }
   }
   new_s[len] = '\0';
 
   if(l1->language) {
-    len = strlen((const char*)l1->language);
+    len = strlen(RASQAL_GOOD_CAST(const char*, l1->language));
     new_lang = RASQAL_MALLOC(char*, len + 1);
     if(!new_lang)
       goto failed;
@@ -500,8 +405,9 @@ rasqal_expression_evaluate_str_prefix_suffix(rasqal_expression *e,
     } else if(e->op == RASQAL_EXPR_STRENDS) {
       b = !memcmp(s1 + len1 - len2, s2, len2);
     } else { /* RASQAL_EXPR_CONTAINS */
-      /* b = (strnstr((const char*)s1, (const char*)s2, len2) != NULL); */
-      b = (strstr((const char*)s1, (const char*)s2) != NULL);
+      /* b = (strnstr(RASQAL_GOOD_CAST(const char*, s1), RASQAL_GOOD_CAST(const char*, s2), len2) != NULL); */
+      b = (strstr(RASQAL_GOOD_CAST(const char*, s1),
+                  RASQAL_GOOD_CAST(const char*, s2)) != NULL);
     }
   }
   
@@ -659,10 +565,11 @@ rasqal_expression_evaluate_concat(rasqal_expression *e,
     arg_literal = rasqal_expression_evaluate2(arg_expr, eval_context, error_p);
     if(arg_literal) {
 
-      if(!dt)
+      if(!dt) {
         /* First datatype URI seen is the result datatype */
-        dt = raptor_uri_copy(arg_literal->datatype);
-      else {
+        if(arg_literal->datatype)
+          dt = raptor_uri_copy(arg_literal->datatype);
+      } else {
         /* Otherwise if all same so far, check the datatype URI for
          * this literal is also the same 
          */
@@ -799,6 +706,7 @@ rasqal_expression_evaluate_strmatch(rasqal_expression *e,
 {
   rasqal_world* world = eval_context->world;
   int b = 0;
+  const unsigned char *l1_str;
   const char *match_string;
   const char *pattern;
   const char *regex_flags;
@@ -810,9 +718,9 @@ rasqal_expression_evaluate_strmatch(rasqal_expression *e,
   if(*error_p || !l1)
     goto failed;
 
-  match_string = (const char*)rasqal_literal_as_counted_string(l1, &match_len, 
-                                                               eval_context->flags,
-                                                               error_p);
+  l1_str = rasqal_literal_as_counted_string(l1, &match_len,
+                                            eval_context->flags, error_p);
+  match_string = RASQAL_GOOD_CAST(const char*, l1_str);
   if(*error_p || !match_string) {
     rasqal_free_literal(l1);
     goto failed;
@@ -834,14 +742,14 @@ rasqal_expression_evaluate_strmatch(rasqal_expression *e,
         rasqal_free_literal(l2);
         goto failed;
       }
-      regex_flags = (const char*)(l3->string);
+      regex_flags = RASQAL_GOOD_CAST(const char*, l3->string);
     }
       
   } else {
     l2 = e->literal;
-    regex_flags = (const char*)(l2->flags);
+    regex_flags = RASQAL_GOOD_CAST(const char*, l2->flags);
   }
-  pattern = (const char*)(l2->string);
+  pattern = RASQAL_GOOD_CAST(const char*, l2->string);
 
   rc = rasqal_regex_match(world, eval_context->locator,
                           pattern, regex_flags,
@@ -849,9 +757,9 @@ rasqal_expression_evaluate_strmatch(rasqal_expression *e,
 
 #ifdef RASQAL_DEBUG
   if(rc >= 0)
-    RASQAL_DEBUG5("regex match returned %s for '%s' against '%s' (flags=%s)\n", rc ? "true" : "false", match_string, pattern, l2->flags ? (char*)l2->flags : "");
+    RASQAL_DEBUG5("regex match returned %s for '%s' against '%s' (flags=%s)\n", rc ? "true" : "false", match_string, pattern, l2->flags ? RASQAL_GOOD_CAST(char*, l2->flags) : "");
   else
-    RASQAL_DEBUG4("regex match returned failed for '%s' against '%s' (flags=%s)\n", match_string, pattern, l2->flags ? (char*)l2->flags : "");
+    RASQAL_DEBUG4("regex match returned failed for '%s' against '%s' (flags=%s)\n", match_string, pattern, l2->flags ? RASQAL_GOOD_CAST(char*, l2->flags) : "");
 #endif
   
   rasqal_free_literal(l1);
@@ -899,7 +807,7 @@ rasqal_expression_evaluate_strbefore(rasqal_expression *e,
   const unsigned char *needle;
   size_t haystack_len;
   size_t needle_len;
-  char *ptr;
+  const char *ptr;
   unsigned char* result;
   size_t result_len;
 
@@ -924,15 +832,22 @@ rasqal_expression_evaluate_strbefore(rasqal_expression *e,
   if(*error_p || !needle)
     goto failed;
 
-  ptr = strstr((const char*)haystack, (const char*)needle);
+  ptr = strstr(RASQAL_GOOD_CAST(const char*, haystack), 
+               RASQAL_GOOD_CAST(const char*, needle));
   if(ptr) {
-    result_len = ptr - (const char*)haystack;
+    result_len = ptr - RASQAL_GOOD_CAST(const char*, haystack);
   } else {
     result_len = 0;
-    haystack = (const unsigned char *)"";
+    haystack = RASQAL_GOOD_CAST(const unsigned char *, "");
   }
 
+  rasqal_free_literal(l1); l1 = NULL;
+  rasqal_free_literal(l2); l2 = NULL;
+
   result = RASQAL_MALLOC(unsigned char*, result_len + 1);
+  if(!result)
+    goto failed;
+
   if(result_len)
     memcpy(result, haystack, result_len);
   result[result_len] = '\0';
@@ -1002,16 +917,23 @@ rasqal_expression_evaluate_strafter(rasqal_expression *e,
   if(*error_p || !needle)
     goto failed;
 
-  ptr = strstr((const char*)haystack, (const char*)needle);
+  ptr = strstr(RASQAL_GOOD_CAST(const char*, haystack),
+               RASQAL_GOOD_CAST(const char*, needle));
   if(ptr) {
     ptr += needle_len;
-    result_len = haystack_len - (ptr - (const char*)haystack);
+    result_len = haystack_len - (ptr - RASQAL_GOOD_CAST(const char*, haystack));
   } else {
     ptr = (const char *)"";
     result_len = 0;
   }
 
+  rasqal_free_literal(l1); l1 = NULL;
+  rasqal_free_literal(l2); l2 = NULL;
+
   result = RASQAL_MALLOC(unsigned char*, result_len + 1);
+  if(!result)
+    goto failed;
+
   if(result_len)
     memcpy(result, ptr, result_len);
   result[result_len] = '\0';
@@ -1050,6 +972,7 @@ rasqal_expression_evaluate_replace(rasqal_expression *e,
                                    int *error_p)
 {
   rasqal_world* world = eval_context->world;
+  const unsigned char *tmp_str;
   const char *match;
   const char *pattern;
   const char *replace;
@@ -1067,23 +990,25 @@ rasqal_expression_evaluate_replace(rasqal_expression *e,
   l1 = rasqal_expression_evaluate2(e->arg1, eval_context, error_p);
   if(*error_p || !l1)
     goto failed;
-  match = (const char*)rasqal_literal_as_counted_string(l1, &match_len, 
-                                                        eval_context->flags,
-                                                        error_p);
+  tmp_str = rasqal_literal_as_counted_string(l1, &match_len, 
+                                             eval_context->flags,
+                                             error_p);
+  match = RASQAL_GOOD_CAST(const char*, tmp_str);
   if(*error_p || !match)
     goto failed;
 
   l2 = rasqal_expression_evaluate2(e->arg2, eval_context, error_p);
   if(*error_p || !l2)
     goto failed;
-  pattern = (const char*)(l2->string);
+  pattern = RASQAL_GOOD_CAST(const char*, l2->string);
 
   l3 = rasqal_expression_evaluate2(e->arg3, eval_context, error_p);
   if(*error_p || !l3)
     goto failed;
-  replace = (const char*)rasqal_literal_as_counted_string(l3, &replace_len, 
-                                                          eval_context->flags,
-                                                          error_p);
+  tmp_str = rasqal_literal_as_counted_string(l3, &replace_len, 
+                                             eval_context->flags,
+                                             error_p);
+  replace = RASQAL_GOOD_CAST(const char*, tmp_str);
   if(*error_p || !replace)
     goto failed;
 
@@ -1092,7 +1017,7 @@ rasqal_expression_evaluate_replace(rasqal_expression *e,
     if(*error_p || !l4)
       goto failed;
 
-    regex_flags = (const char*)(l4->string);
+    regex_flags = RASQAL_GOOD_CAST(const char*, l4->string);
   }
 
   result_s = rasqal_regex_replace(world, eval_context->locator,
@@ -1102,12 +1027,13 @@ rasqal_expression_evaluate_replace(rasqal_expression *e,
                                   replace, replace_len,
                                   &result_len);
   
-  RASQAL_DEBUG6("regex replace returned %s for '%s' from '%s' to '%s' (flags=%s)\n", result_s ? result_s : "NULL", match, pattern, replace, regex_flags ? (char*)regex_flags : "");
+  RASQAL_DEBUG6("regex replace returned %s for '%s' from '%s' to '%s' (flags=%s)\n", result_s ? result_s : "NULL", match, pattern, replace, regex_flags ? RASQAL_GOOD_CAST(char*, regex_flags) : "");
   
   if(!result_s)
     goto failed;
 
-  result = rasqal_new_string_literal(world, (const unsigned char*)result_s,
+  result = rasqal_new_string_literal(world, 
+                                     RASQAL_GOOD_CAST(const unsigned char*, result_s),
                                      l1->language, l1->datatype, NULL);
   l1->language = NULL;
   l1->datatype = NULL;
