@@ -121,7 +121,7 @@ typedef struct
 
 
 static const unsigned char* const rs_namespace_uri_string =
-  (const unsigned char*)"http://www.w3.org/2001/sw/DataAccess/tests/result-set#";
+  RASQAL_GOOD_CAST(const unsigned char*, "http://www.w3.org/2001/sw/DataAccess/tests/result-set#");
 
 
 static int
@@ -155,33 +155,33 @@ rasqal_query_results_rdf_init(rasqal_query_results_formatter* formatter,
   context->rdf_type_uri = 
     raptor_new_uri_from_uri_local_name(raptor_world_ptr,
                                        context->rdf_ns_uri,
-                                       (const unsigned char*)"type");
+                                       RASQAL_GOOD_CAST(const unsigned char*, "type"));
   context->rs_variable_uri =
     raptor_new_uri_from_uri_local_name(raptor_world_ptr,
                                        context->rs_ns_uri,
-                                       (const unsigned char*)"variable");
+                                       RASQAL_GOOD_CAST(const unsigned char*, "variable"));
   context->rs_value_uri =
     raptor_new_uri_from_uri_local_name(raptor_world_ptr,
                                        context->rs_ns_uri,
-                                       (const unsigned char*)"value");
+                                       RASQAL_GOOD_CAST(const unsigned char*, "value"));
   context->rs_solution_uri =
     raptor_new_uri_from_uri_local_name(raptor_world_ptr,
                                        context->rs_ns_uri,
-                                       (const unsigned char*)"solution");
+                                       RASQAL_GOOD_CAST(const unsigned char*, "solution"));
   context->rs_binding_uri =
     raptor_new_uri_from_uri_local_name(raptor_world_ptr,
                                        context->rs_ns_uri,
-                                       (const unsigned char*)"binding");
+                                       RASQAL_GOOD_CAST(const unsigned char*, "binding"));
   context->rs_resultVariable_uri =
     raptor_new_uri_from_uri_local_name(raptor_world_ptr,
                                        context->rs_ns_uri,
-                                       (const unsigned char*)"resultVariable");
+                                       RASQAL_GOOD_CAST(const unsigned char*, "resultVariable"));
 
   /* Classes */
   context->rs_ResultSet_uri =
     raptor_new_uri_from_uri_local_name(raptor_world_ptr,
                                        context->rs_ns_uri,
-                                       (const unsigned char*)"ResultSet");
+                                       RASQAL_GOOD_CAST(const unsigned char*, "ResultSet"));
 
   return 0;
 }
@@ -374,7 +374,7 @@ rasqal_query_results_rdf_write(rasqal_query_results_formatter* formatter,
             statement.object = raptor_new_term_from_literal(raptor_world_ptr,
                                                             l->string,
                                                             l->datatype,
-                                                            (const unsigned char*)l->language);
+                                                            RASQAL_GOOD_CAST(const unsigned char*, l->language));
             break;
         case RASQAL_LITERAL_PATTERN:
         case RASQAL_LITERAL_QNAME:
@@ -385,6 +385,7 @@ rasqal_query_results_rdf_write(rasqal_query_results_formatter* formatter,
         case RASQAL_LITERAL_FLOAT:
         case RASQAL_LITERAL_VARIABLE:
         case RASQAL_LITERAL_DECIMAL:
+        case RASQAL_LITERAL_DATE:
         case RASQAL_LITERAL_DATETIME:
         case RASQAL_LITERAL_INTEGER_SUBTYPE:
 
@@ -508,24 +509,24 @@ rasqal_rowsource_rdf_process(rasqal_rowsource_rdf_context* con)
 
   uri = raptor_new_uri_from_uri_local_name(con->raptor_world_ptr,
                                            rdf_ns_uri,
-                                           (const unsigned char*)"type");
+                                           RASQAL_GOOD_CAST(const unsigned char*, "type"));
   predicate_uri_literal = rasqal_new_uri_literal(con->world, uri);
 
   raptor_free_uri(rdf_ns_uri); rdf_ns_uri = NULL;
 
   uri = raptor_new_uri_from_uri_local_name(con->raptor_world_ptr,
                                            con->rs_uri,
-                                           (const unsigned char*)"ResultSet");
+                                           RASQAL_GOOD_CAST(const unsigned char*, "ResultSet"));
   object_uri_literal = rasqal_new_uri_literal(con->world, uri);
 
   uri = raptor_new_uri_from_uri_local_name(con->raptor_world_ptr,
                                            con->rs_uri,
-                                           (const unsigned char*)"variable");
+                                           RASQAL_GOOD_CAST(const unsigned char*, "variable"));
   variable_predicate = rasqal_new_uri_literal(con->world, uri);
   
   uri = raptor_new_uri_from_uri_local_name(con->raptor_world_ptr,
                                            con->rs_uri,
-                                           (const unsigned char*)"value");
+                                           RASQAL_GOOD_CAST(const unsigned char*, "value"));
   value_predicate = rasqal_new_uri_literal(con->world, uri);
   
 
@@ -544,7 +545,7 @@ rasqal_rowsource_rdf_process(rasqal_rowsource_rdf_context* con)
   if(!resultSet_node)
     return 0;
   
-#if RASQAL_DEBUG > 2
+#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 2
   RASQAL_DEBUG1("Got result set node ");
   rasqal_literal_print(resultSet_node, DEBUG_FH);
   fputc('\n', DEBUG_FH);
@@ -555,7 +556,7 @@ rasqal_rowsource_rdf_process(rasqal_rowsource_rdf_context* con)
   /* find variable names from all ?var := getTargets(?rs, rs:resultVariable) */
   uri = raptor_new_uri_from_uri_local_name(con->raptor_world_ptr,
                                            con->rs_uri,
-                                           (const unsigned char*)"resultVariable");
+                                           RASQAL_GOOD_CAST(const unsigned char*, "resultVariable"));
   predicate_uri_literal = rasqal_new_uri_literal(con->world, uri);
 
   solution_iter = rasqal_dataset_get_targets_iterator(con->ds,
@@ -566,7 +567,7 @@ rasqal_rowsource_rdf_process(rasqal_rowsource_rdf_context* con)
     if(!solution_node)
       break;
 
-#if RASQAL_DEBUG > 2
+#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 2
     RASQAL_DEBUG1("Got variable node ");
     rasqal_literal_print(solution_node, DEBUG_FH);
     fputc('\n', DEBUG_FH);
@@ -579,18 +580,21 @@ rasqal_rowsource_rdf_process(rasqal_rowsource_rdf_context* con)
       name = rasqal_literal_as_counted_string(solution_node, &len, 0, NULL);
       if(name) {
         unsigned char* var_name;
-        rasqal_variable *v;
 
         var_name = RASQAL_MALLOC(unsigned char*, len + 1);
-        memcpy(var_name, name, len + 1);
+        if(var_name) {
+          rasqal_variable *v;
 
-        v = rasqal_variables_table_add(con->vars_table,
-                                       RASQAL_VARIABLE_TYPE_NORMAL,
-                                       var_name, NULL);
-        if(v) {
-          rasqal_rowsource_add_variable(con->rowsource, v);
-          /* above function takes a reference to v */
-          rasqal_free_variable(v);
+          memcpy(var_name, name, len + 1);
+          
+          v = rasqal_variables_table_add(con->vars_table,
+                                         RASQAL_VARIABLE_TYPE_NORMAL,
+                                         var_name, NULL);
+          if(v) {
+            rasqal_rowsource_add_variable(con->rowsource, v);
+            /* above function takes a reference to v */
+            rasqal_free_variable(v);
+          }
         }
       }
     }
@@ -603,14 +607,14 @@ rasqal_rowsource_rdf_process(rasqal_rowsource_rdf_context* con)
 
   uri = raptor_new_uri_from_uri_local_name(con->raptor_world_ptr,
                                            con->rs_uri,
-                                           (const unsigned char*)"binding");
+                                           RASQAL_GOOD_CAST(const unsigned char*, "binding"));
   binding_uri_literal = rasqal_new_uri_literal(con->world, uri);
 
 
   /* for each solution node ?sol := getTargets(?rs, rs:solution) */
   uri = raptor_new_uri_from_uri_local_name(con->raptor_world_ptr,
                                            con->rs_uri,
-                                           (const unsigned char*)"solution");
+                                           RASQAL_GOOD_CAST(const unsigned char*, "solution"));
   predicate_uri_literal = rasqal_new_uri_literal(con->world, uri);
 
   solution_iter = rasqal_dataset_get_targets_iterator(con->ds,
@@ -624,7 +628,7 @@ rasqal_rowsource_rdf_process(rasqal_rowsource_rdf_context* con)
     if(!solution_node)
       break;
 
-#if RASQAL_DEBUG > 2
+#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 2
     RASQAL_DEBUG1("Got solution node (row) ");
     rasqal_literal_print(solution_node, DEBUG_FH);
     fputc('\n', DEBUG_FH);
@@ -645,7 +649,7 @@ rasqal_rowsource_rdf_process(rasqal_rowsource_rdf_context* con)
       if(!binding_node)
         break;
 
-#if RASQAL_DEBUG > 2
+#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 2
       RASQAL_DEBUG1("  Got binding node ");
       rasqal_literal_print(binding_node, DEBUG_FH);
       fputc('\n', DEBUG_FH);
@@ -661,7 +665,7 @@ rasqal_rowsource_rdf_process(rasqal_rowsource_rdf_context* con)
                                                 binding_node,
                                                 value_predicate);
 
-#if RASQAL_DEBUG > 2
+#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 2
       RASQAL_DEBUG1("    Variable: ");
       rasqal_literal_print(var_literal, DEBUG_FH);
       fputs(" Value: ", DEBUG_FH);
@@ -831,7 +835,7 @@ rasqal_query_results_rdfxml_recognise_syntax(rasqal_query_results_format_factory
 const char *mime_type)
 {
 
-  if(suffix && !strcmp((const char*)suffix, "rdf"))
+  if(suffix && !strcmp(RASQAL_GOOD_CAST(const char*, suffix), "rdf"))
     return 5;
   
   return 0;
@@ -1002,8 +1006,8 @@ rasqal_query_results_turtle_recognise_syntax(rasqal_query_results_format_factory
                                              const unsigned char *suffix,
 const char *mime_type)
 {
-  if(suffix && (!strcmp((const char*)suffix, "ttl") ||
-                !strcmp((const char*)suffix, "n3")))
+  if(suffix && (!strcmp(RASQAL_GOOD_CAST(const char*, suffix), "ttl") ||
+                !strcmp(RASQAL_GOOD_CAST(const char*, suffix), "n3")))
     return 7;
   
   return 0;
