@@ -48,7 +48,7 @@ static int rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter*
 static rasqal_rowsource* rasqal_query_results_get_rowsource_sparql_xml(rasqal_query_results_formatter* formatter, rasqal_world *world, rasqal_variables_table* vars_table, raptor_iostream *iostr, raptor_uri *base_uri, unsigned int flags);
 
 
-#if RASQAL_DEBUG > 1
+#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 1
 #define TRACE_XML 1
 #else
 #undef TRACE_XML
@@ -97,15 +97,18 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
   raptor_xml_element *variable_element=NULL;
   raptor_qname **attrs=NULL;
   int i;
+  rasqal_query_results_type type;
 
-  if(!rasqal_query_results_is_bindings(results) &&
-     !rasqal_query_results_is_boolean(results)) {
+  type = rasqal_query_results_get_type(results);
+
+  if(type != RASQAL_QUERY_RESULTS_BINDINGS &&
+     type != RASQAL_QUERY_RESULTS_BOOLEAN) {
     rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR,
                             NULL,
-                            "Can only write XML format v3 for variable binding and boolean results");
+                            "Cannot write XML format v3 for %s query result format",
+                            rasqal_query_results_type_label(type));
     return 1;
   }
-  
 
   nstack = raptor_new_namespaces(world->raptor_world_ptr, 1);
   if(!nstack)
@@ -119,30 +122,30 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
 
   res_ns=raptor_new_namespace(nstack,
                               NULL,
-                              (const unsigned char*)"http://www.w3.org/2005/sparql-results#",
+                              RASQAL_GOOD_CAST(const unsigned char*, "http://www.w3.org/2005/sparql-results#"),
                               0);
   if(!res_ns)
     goto tidy;
 
   sparql_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                  (const unsigned char*)"sparql",
+                                                                  RASQAL_GOOD_CAST(const unsigned char*, "sparql"),
                                                                   NULL, base_uri);
   if(!sparql_element)
     goto tidy;
 
   raptor_xml_writer_start_element(xml_writer, sparql_element);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
   /*   <head> */
   element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                            (const unsigned char*)"head",
+                                                            RASQAL_GOOD_CAST(const unsigned char*, "head"),
                                                             NULL, base_uri);
   if(!element1)
     goto tidy;
 
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"  ", 2);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "  "), 2);
   raptor_xml_writer_start_element(xml_writer, element1);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
   
   if(rasqal_query_results_is_bindings(results)) {
     for(i=0; 1; i++) {
@@ -153,7 +156,7 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
       
       /*     <variable name="x"/> */
       variable_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                        (const unsigned char*)"variable",
+                                                                        RASQAL_GOOD_CAST(const unsigned char*,"variable"),
                                                                         NULL, base_uri);
       if(!variable_element)
         goto tidy;
@@ -163,8 +166,8 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
         goto tidy;
       attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                             res_ns,
-                                                            (const unsigned char*)"name",
-                                                            (const unsigned char*)name); /* attribute value */
+                                                            RASQAL_GOOD_CAST(const unsigned char*,"name"),
+                                                            RASQAL_GOOD_CAST(const unsigned char*, name)); /* attribute value */
       if(!attrs[0]) {
         raptor_free_memory((void*)attrs);
         goto tidy;
@@ -172,9 +175,9 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
 
       raptor_xml_element_set_attributes(variable_element, attrs, 1);
       
-      raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"    ", 4);
+      raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "    "), 4);
       raptor_xml_writer_empty_element(xml_writer, variable_element);
-      raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+      raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
       
       raptor_free_xml_element(variable_element);
       variable_element=NULL;
@@ -185,9 +188,9 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
 
     
   /*   </head> */
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"  ", 2);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "  "), 2);
   raptor_xml_writer_end_element(xml_writer, element1);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
   
   raptor_free_xml_element(element1);
   element1=NULL;
@@ -196,19 +199,19 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
   /* Boolean Results */
   if(rasqal_query_results_is_boolean(results)) {
     result_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                    (const unsigned char*)"boolean",
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, "boolean"),
                                                                     NULL, base_uri);
     if(!result_element)
       goto tidy;
 
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"  ", 2);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "  "), 2);
     raptor_xml_writer_start_element(xml_writer, result_element);
     if(rasqal_query_results_get_boolean(results))
       raptor_xml_writer_raw(xml_writer, RASQAL_XSD_BOOLEAN_TRUE);
     else
       raptor_xml_writer_raw(xml_writer, RASQAL_XSD_BOOLEAN_FALSE);
     raptor_xml_writer_end_element(xml_writer, result_element);
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
     goto results3done;
   }
@@ -218,28 +221,28 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
 
   /*   <results> */
   results_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                   (const unsigned char*)"results",
+                                                                   RASQAL_GOOD_CAST(const unsigned char*, "results"),
                                                                    NULL, base_uri);
   if(!results_element)
     goto tidy;
 
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"  ", 2);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "  "), 2);
   raptor_xml_writer_start_element(xml_writer, results_element);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
 
   /* declare result element for later multiple use */
   result_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                  (const unsigned char*)"result",
+                                                                  RASQAL_GOOD_CAST(const unsigned char*, "result"),
                                                                   NULL, base_uri);
   if(!result_element)
     goto tidy;
 
   while(!rasqal_query_results_finished(results)) {
     /*     <result> */
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"    ", 4);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "    "), 4);
     raptor_xml_writer_start_element(xml_writer, result_element);
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
     for(i=0; i<rasqal_query_results_get_bindings_count(results); i++) {
       const unsigned char *name=rasqal_query_results_get_binding_name(results, i);
@@ -247,7 +250,7 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
 
       /*       <binding> */
       binding_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                       (const unsigned char*)"binding",
+                                                                       RASQAL_GOOD_CAST(const unsigned char*, "binding"),
                                                                        NULL, base_uri);
       if(!binding_element)
         goto tidy;
@@ -257,7 +260,7 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
         goto tidy;
       attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                             res_ns,
-                                                            (const unsigned char*)"name",
+                                                            RASQAL_GOOD_CAST(const unsigned char*, "name"),
                                                             name);
 
       if(!attrs[0]) {
@@ -268,12 +271,12 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
       raptor_xml_element_set_attributes(binding_element, attrs, 1);
       
 
-      raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"      ", 6);
+      raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "      "), 6);
       raptor_xml_writer_start_element(xml_writer, binding_element);
 
       if(!l) {
         element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                  (const unsigned char*)"unbound",
+                                                                  RASQAL_GOOD_CAST(const unsigned char*, "unbound"),
                                                                   NULL, base_uri);
         if(!element1)
           goto tidy;
@@ -282,33 +285,33 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
       } else switch(l->type) {
         case RASQAL_LITERAL_URI:
           element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                    (const unsigned char*)"uri",
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, "uri"),
                                                                     NULL, base_uri);
           if(!element1)
             goto tidy;
           
           raptor_xml_writer_start_element(xml_writer, element1);
-          raptor_xml_writer_cdata(xml_writer, (const unsigned char*)raptor_uri_as_string(l->value.uri));
+          raptor_xml_writer_cdata(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, raptor_uri_as_string(l->value.uri)));
           raptor_xml_writer_end_element(xml_writer, element1);
 
           break;
 
         case RASQAL_LITERAL_BLANK:
           element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                    (const unsigned char*)"bnode",
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, "bnode"),
                                                                     NULL, base_uri);
           if(!element1)
             goto tidy;
           
           raptor_xml_writer_start_element(xml_writer, element1);
-          raptor_xml_writer_cdata(xml_writer, (const unsigned char*)l->string);
+          raptor_xml_writer_cdata(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, l->string));
           raptor_xml_writer_end_element(xml_writer, element1);
           break;
 
         case RASQAL_LITERAL_STRING:
         case RASQAL_LITERAL_UDT:
           element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                    (const unsigned char*)"literal",
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, "literal"),
                                                                     NULL, base_uri);
           if(!element1)
             goto tidy;
@@ -320,13 +323,13 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
 
             if(l->language)
               attrs[0]=raptor_new_qname(nstack,
-                                        (const unsigned char*)"xml:lang",
-                                        (const unsigned char*)l->language);
+                                        RASQAL_GOOD_CAST(const unsigned char*, "xml:lang"),
+                                        RASQAL_GOOD_CAST(const unsigned char*, l->language));
             else
               attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                                     res_ns,
-                                                                    (const unsigned char*)"datatype",
-                                                                    (const unsigned char*)raptor_uri_as_string(l->datatype));
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, "datatype"),
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, raptor_uri_as_string(l->datatype)));
             if(!attrs[0]) {
               raptor_free_memory((void*)attrs);
               goto tidy;
@@ -340,7 +343,7 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
 
 
           raptor_xml_writer_cdata_counted(xml_writer,
-                                          (const unsigned char*)l->string, 
+                                          RASQAL_GOOD_CAST(const unsigned char*, l->string),
                                           l->string_len);
 
           raptor_xml_writer_end_element(xml_writer, element1);
@@ -355,6 +358,7 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
         case RASQAL_LITERAL_FLOAT:
         case RASQAL_LITERAL_VARIABLE:
         case RASQAL_LITERAL_DECIMAL:
+        case RASQAL_LITERAL_DATE:
         case RASQAL_LITERAL_DATETIME:
         case RASQAL_LITERAL_INTEGER_SUBTYPE:
 
@@ -374,29 +378,29 @@ rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
 
       /*       </binding> */
       raptor_xml_writer_end_element(xml_writer, binding_element);
-      raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+      raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
       
       raptor_free_xml_element(binding_element);
       binding_element=NULL;
     }
 
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"    ", 4);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "    "), 4);
     raptor_xml_writer_end_element(xml_writer, result_element);
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
     
     rasqal_query_results_next(results);
   }
 
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"  ", 2);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "  "), 2);
   raptor_xml_writer_end_element(xml_writer, results_element);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
   results3done:
 
   rc=0;
 
   raptor_xml_writer_end_element(xml_writer, sparql_element);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
   tidy:
   if(element1)
@@ -526,7 +530,7 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
   name=raptor_xml_element_get_name(xml_element);
 
   for(i=STATE_first; i <= STATE_last; i++) {
-    if(!strcmp((const char*)raptor_qname_get_local_name(name),
+    if(!strcmp(RASQAL_GOOD_CAST(const char*, raptor_qname_get_local_name(name)),
                sparql_xml_element_names[i])) {
       state=(rasqal_sparql_xml_read_state)i;
       con->state=state;
@@ -555,26 +559,24 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
   
   if(attr_count > 0) {
     raptor_qname** attrs=raptor_xml_element_get_attributes(xml_element);
-    for(i=0; i < attr_count; i++) {
+    for(i = 0; i < attr_count; i++) {
+      const char* local_name = RASQAL_GOOD_CAST(const char*, raptor_qname_get_local_name(attrs[i]));
 #ifdef TRACE_XML
       if(con->trace) {
         pad(stderr, con->depth+1);
         fprintf(stderr, "Attribute %s='%s'\n",
-                raptor_qname_get_local_name(attrs[i]),
-                raptor_qname_get_value(attrs[i]));
+                local_name, raptor_qname_get_value(attrs[i]));
       }
 #endif
-      if(!strcmp((const char*)raptor_qname_get_local_name(attrs[i]),
-                 "name"))
-        con->name=(const char*)raptor_qname_get_counted_value(attrs[i],
-                                                             &con->name_length);
-      else if(!strcmp((const char*)raptor_qname_get_local_name(attrs[i]),
-                      "datatype"))
-        con->datatype=(const char*)raptor_qname_get_value(attrs[i]);
+
+      if(!strcmp(local_name, "name"))
+        con->name = RASQAL_GOOD_CAST(const char*, raptor_qname_get_counted_value(attrs[i], &con->name_length));
+      else if(!strcmp(local_name, "datatype"))
+        con->datatype = RASQAL_GOOD_CAST(const char*, raptor_qname_get_value(attrs[i]));
     }
   }
   if(raptor_xml_element_get_language(xml_element)) {
-    con->language=(const char*)raptor_xml_element_get_language(xml_element);
+    con->language = RASQAL_GOOD_CAST(const char*, raptor_xml_element_get_language(xml_element));
 #ifdef TRACE_XML
     if(con->trace) {
       pad(stderr, con->depth+1);
@@ -612,7 +614,7 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
       break;
       
     case STATE_binding:
-      con->result_offset = rasqal_rowsource_get_variable_offset_by_name(con->rowsource, (const unsigned char*)con->name);
+      con->result_offset = rasqal_rowsource_get_variable_offset_by_name(con->rowsource, RASQAL_GOOD_CAST(const unsigned char*, con->name));
       break;
       
     case STATE_sparql:
@@ -672,7 +674,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
   name=raptor_xml_element_get_name(xml_element);
 
   for(i=STATE_first; i <= STATE_last; i++) {
-    if(!strcmp((const char*)raptor_qname_get_local_name(name),
+    if(!strcmp(RASQAL_GOOD_CAST(const char*, raptor_qname_get_local_name(name)),
                sparql_xml_element_names[i])) {
       state=(rasqal_sparql_xml_read_state)i;
       con->state=state;
@@ -713,7 +715,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
         else
           memcpy(lvalue, con->value, con->value_len + 1);
         if(con->datatype)
-          datatype_uri = raptor_new_uri(con->world->raptor_world_ptr, (const unsigned char*)con->datatype);
+          datatype_uri = raptor_new_uri(con->world->raptor_world_ptr, RASQAL_GOOD_CAST(const unsigned char*, con->datatype));
         if(con->language) {
           size_t language_len = strlen(con->language);
           language_str = RASQAL_MALLOC(char*, language_len + 1);
@@ -746,7 +748,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
       if(1) {
         raptor_uri* uri;
         rasqal_literal* l;
-        uri = raptor_new_uri(con->world->raptor_world_ptr, (const unsigned char*)con->value);
+        uri = raptor_new_uri(con->world->raptor_world_ptr, RASQAL_GOOD_CAST(const unsigned char*, con->value));
         l = rasqal_new_uri_literal(con->world, uri);
         rasqal_row_set_value_at(con->row, con->result_offset, l);
         rasqal_free_literal(l);
@@ -841,11 +843,11 @@ rasqal_rowsource_sparql_xml_process(rasqal_rowsource_sparql_xml_context* con)
   while(!raptor_iostream_read_eof(con->iostr)) {
     size_t read_len;
     
-    read_len = raptor_iostream_read_bytes((char*)con->buffer, 1,
+    read_len = raptor_iostream_read_bytes(RASQAL_GOOD_CAST(char*, con->buffer), 1,
                                           FILE_READ_BUF_SIZE,
                                           con->iostr);
     if(read_len > 0) {
-      RASQAL_DEBUG2("processing %d bytes\n", (int)read_len);
+      RASQAL_DEBUG2("processing %d bytes\n", RASQAL_GOOD_CAST(int, read_len));
       raptor_sax2_parse_chunk(con->sax2, con->buffer, read_len, 0);
     }
     
@@ -920,8 +922,8 @@ static const rasqal_rowsource_handler rasqal_rowsource_sparql_xml_handler={
  * @iostr: #raptor_iostream to read the query results from
  * @base_uri: #raptor_uri base URI of the input format
  *
- * Read the fourth version of the SPARQL XML query results format from an
- * iostream in a format returning a rwosurce - INTERNAL.
+ * INTERNAL - Read the fourth version of the SPARQL XML query results
+ * format from an iostream in a format returning a rowsource.
  * 
  * Return value: a new rasqal_rowsource or NULL on failure
  **/
@@ -981,7 +983,7 @@ rasqal_query_results_xml_recognise_syntax(rasqal_query_results_format_factory* f
 const char *mime_type)
 {
 
-  if(suffix && !strcmp((const char*)suffix, "srx"))
+  if(suffix && !strcmp(RASQAL_GOOD_CAST(const char*, suffix), "srx"))
     return 8;
   
   return 0;
