@@ -86,54 +86,10 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
                                   const raptor_statement *statement)
 {
   rasqal_literal *s, *p, *o;
-  raptor_uri *uri;
-  unsigned char *new_str;
-  size_t len;
 
-  /* subject */
-  if(statement->subject->type == RAPTOR_TERM_TYPE_BLANK) {
-    len = statement->subject->value.blank.string_len;
-    new_str = RASQAL_MALLOC(unsigned char*, len + 1);
-    memcpy(new_str, statement->subject->value.blank.string, len + 1);
-    s = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_str);
-  } else {
-    uri = raptor_uri_copy((raptor_uri*)statement->subject->value.uri);
-    s = rasqal_new_uri_literal(world, uri);
-  }
-
-  /* predicate */
-  uri = raptor_uri_copy((raptor_uri*)statement->predicate->value.uri);
-  p = rasqal_new_uri_literal(world, uri);
-  
-  /* object */
-  if(statement->object->type == RAPTOR_TERM_TYPE_LITERAL) {
-    char *language = NULL;
-
-    len = statement->object->value.literal.string_len;
-    new_str = RASQAL_MALLOC(unsigned char*, len + 1);
-    memcpy(new_str, statement->object->value.literal.string, len + 1);
-
-    if(statement->object->value.literal.language) {
-      len = statement->object->value.literal.language_len;
-      language = RASQAL_MALLOC(char*, len + 1);
-      memcpy(language, statement->object->value.literal.language, len + 1);
-    }
-
-    if(statement->object->value.literal.datatype)
-      uri = raptor_uri_copy(statement->object->value.literal.datatype);
-    else
-      uri = NULL;
-
-    o = rasqal_new_string_literal(world, new_str, language, uri, NULL);
-  } else if(statement->object->type == RAPTOR_TERM_TYPE_BLANK) {
-    len = statement->object->value.blank.string_len;
-    new_str = RASQAL_MALLOC(unsigned char*, len + 1);
-    memcpy(new_str, statement->object->value.blank.string, len + 1);
-    o = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_str);
-  } else {
-    uri = raptor_uri_copy((raptor_uri*)statement->object->value.uri);
-    o = rasqal_new_uri_literal(world, uri);
-  }
+  s = rasqal_new_literal_from_term(world, statement->subject);
+  p = rasqal_new_literal_from_term(world, statement->predicate);
+  o = rasqal_new_literal_from_term(world, statement->object);
 
   return rasqal_new_triple(s, p, o);
 }
@@ -178,7 +134,7 @@ rasqal_raptor_generate_id_handler(void *user_data,
 
   if(user_bnodeid) {
     unsigned char *mapped_id;
-    size_t user_bnodeid_len = strlen((const char*)user_bnodeid);
+    size_t user_bnodeid_len = strlen(RASQAL_GOOD_CAST(const char*, user_bnodeid));
     
     mapped_id = RASQAL_MALLOC(unsigned char*, 
                               rtsc->mapped_id_base_len + 1 + user_bnodeid_len + 1);
@@ -191,7 +147,7 @@ rasqal_raptor_generate_id_handler(void *user_data,
     return mapped_id;
   }
   
-  return rasqal_query_get_genid(rtsc->query, (const unsigned char*)"genid", -1);
+  return rasqal_query_get_genid(rtsc->query, RASQAL_GOOD_CAST(const unsigned char*, "genid"), -1);
 }
 
 
@@ -272,9 +228,9 @@ rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
     }
 
     rtsc->mapped_id_base = rasqal_query_get_genid(rdf_query,
-                                                  (const unsigned char*)"graphid",
+                                                  RASQAL_GOOD_CAST(const unsigned char*, "graphid"),
                                                   i);
-    rtsc->mapped_id_base_len = strlen((const char*)rtsc->mapped_id_base);
+    rtsc->mapped_id_base_len = strlen(RASQAL_GOOD_CAST(const char*, rtsc->mapped_id_base));
 
     parser_name = dg->format_name;
     if(parser_name) {
@@ -360,7 +316,7 @@ rasqal_raptor_triple_match(rasqal_world* world,
 {
   int rc = 0;
   
-#if RASQAL_DEBUG > 1
+#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 1
   RASQAL_DEBUG1("\ntriple ");
   rasqal_triple_print(triple, stderr);
   fputs("\nmatch  ", stderr);
@@ -413,7 +369,7 @@ rasqal_raptor_triple_match(rasqal_world* world,
   rc = 1;
   done:
   
-#if RASQAL_DEBUG > 1
+#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 1
   RASQAL_DEBUG2("result: %s\n", (rc ? "match" : "no match"));
 #endif
 
