@@ -67,7 +67,8 @@
  *
  * INTERNAL - Test if a string matches a regex pattern.
  *
- * Intended to be used for RDQL string matches/does not match regex operators =~ and !~
+ * Intended to be used for executing #RASQAL_EXPR_STR_MATCH and
+ * #RASQAL_EXPR_STR_NMATCH operations (unused: formerly RDQL)
  *
  * Return value: <0 on error, 0 for no match, >0 for match
  *
@@ -102,7 +103,7 @@ rasqal_regex_match(rasqal_world* world, raptor_locator* locator,
   if(flag_i)
     compile_options |= PCRE_CASELESS;
     
-  re = pcre_compile((const char*)pattern, compile_options,
+  re = pcre_compile(RASQAL_GOOD_CAST(const char*, pattern), compile_options,
                     &re_error, &erroffset, NULL);
   if(!re) {
     rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR, locator,
@@ -111,7 +112,8 @@ rasqal_regex_match(rasqal_world* world, raptor_locator* locator,
   } else {
     rc = pcre_exec(re, 
                    NULL, /* no study */
-                   subject, (int)subject_len,
+                   subject,
+                   RASQAL_BAD_CAST(int, subject_len), /* PCRE API is an int */
                    0 /* startoffset */,
                    exec_options /* options */,
                    NULL, 0 /* ovector, ovecsize - no matches wanted */
@@ -133,14 +135,14 @@ rasqal_regex_match(rasqal_world* world, raptor_locator* locator,
   if(flag_i)
     compile_options |= REG_ICASE;
     
-  rc = regcomp(&reg, (const char*)pattern, compile_options);
+  rc = regcomp(&reg, RASQAL_GOOD_CAST(const char*, pattern), compile_options);
   if(rc) {
     rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR,
                             locator,
                             "Regex compile of '%s' failed", pattern);
     rc = -1;
   } else {
-    rc = regexec(&reg, (const char*)subject, 
+    rc = regexec(&reg, RASQAL_GOOD_CAST(const char*, subject),
                  0, NULL, /* nmatch, regmatch_t pmatch[] - no matches wanted */
                  exec_options /* eflags */
                  );
@@ -245,7 +247,8 @@ rasqal_regex_replace_pcre(rasqal_world* world, raptor_locator* locator,
 
     stringcount = pcre_exec(re,
                             NULL, /* no study */
-                            subject, (int)subject_len,
+                            subject,
+                            RASQAL_BAD_CAST(int, subject_len), /* PCRE API is an int */
                             startoffset,
                             options | matched_empty_options,
                             ovector, ovecsize);
@@ -485,7 +488,7 @@ rasqal_regex_replace_posix(rasqal_world* world, raptor_locator* locator,
     int rc;
     const char *subject_piece = subject + startoffset;
 
-    rc = regexec(&reg, (const char*)subject_piece,
+    rc = regexec(&reg, RASQAL_GOOD_CAST(const char*, subject_piece),
                  capture_count, pmatch,
                  options /* eflags */
                  );
@@ -656,6 +659,9 @@ rasqal_regex_replace_posix(rasqal_world* world, raptor_locator* locator,
 
 
   failed:
+  if(result)
+    RASQAL_FREE(char*, result);
+
   RASQAL_FREE(regmatch_t*, pmatch);
   
   return NULL;
@@ -664,19 +670,19 @@ rasqal_regex_replace_posix(rasqal_world* world, raptor_locator* locator,
 
 
 
-/*
+/**
  * rasqal_regex_replace:
  * @world: world
  * @locator: locator
  * @pattern: regex pattern
- * @options: regex flags string
+ * @regex_flags: regex flags string
  * @subject: input string
  * @subject_len: input string length
  * @replace: replacement string
  * @replace_len: Length of replacement string
  * @result_len_p: pointer to store result length (output)
  *
- * INTERNAL - Replace all copies of matches to a pattern with a replacement with subsitution
+ * Replace all copies of matches to a pattern with a replacement with subsitution
  *
  * Intended to be used for SPARQL 1.1 REPLACE() implementation.
  *
@@ -772,7 +778,7 @@ rasqal_regex_replace(rasqal_world* world, raptor_locator* locator,
   return result_s;
 }
 
-#endif
+#endif /* not STANDALONE */
 
 
 #ifdef STANDALONE
@@ -787,9 +793,9 @@ int
 main(int argc, char *argv[])
 {
   rasqal_world* world;
-  raptor_locator* locator;
   const char *program = rasqal_basename(argv[0]);
 #ifdef RASQAL_REGEX_PCRE
+  raptor_locator* locator = NULL;
   int test = 0;
 #endif
   int failures = 0;
@@ -801,8 +807,6 @@ main(int argc, char *argv[])
     goto tidy;
   }
     
-  locator = NULL;
-
 #if defined(RASQAL_REGEX_POSIX) || defined(RASQAL_REGEX_NONE)
     fprintf(stderr,
             "%s: WARNING: Cannot only run regex tests with PCRE regexes\n",
@@ -816,8 +820,8 @@ main(int argc, char *argv[])
     const char* pattern = "[^a-z0-9]";
     const char* replace = "-";
     const char* expected_result = "abcd1234--";
-    size_t subject_len = strlen((const char*)subject);
-    size_t replace_len = strlen((const char*)replace);
+    size_t subject_len = strlen(RASQAL_GOOD_CAST(const char*, subject));
+    size_t replace_len = strlen(RASQAL_GOOD_CAST(const char*, replace));
     char* result;
     size_t result_len = 0;
     
@@ -849,4 +853,4 @@ main(int argc, char *argv[])
 
   return failures;
 }
-#endif
+#endif /* STANDALONE */
