@@ -191,7 +191,7 @@ rasqal_get_query_results_formatter_factory(rasqal_world* world,
 
     if(uri && factory->desc.uri_strings) {
       int j;
-      const char* uri_string = (const char*)raptor_uri_as_string(uri);
+      const char* uri_string = RASQAL_GOOD_CAST(const char*, raptor_uri_as_string(uri));
       const char* factory_uri_string = NULL;
       
       for(j = 0;
@@ -498,8 +498,8 @@ rasqal_query_results_formatter_read(rasqal_world *world,
                                     rasqal_query_results* results,
                                     raptor_uri *base_uri)
 {
-  rasqal_rowsource* rowsource = NULL;
-  rasqal_variables_table* vars_table;
+  rasqal_query_results_type type;
+  int rc = 0;
   
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(iostr, raptor_iostream, 1);
@@ -507,26 +507,39 @@ rasqal_query_results_formatter_read(rasqal_world *world,
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(results, rasqal_query_results, 1);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(base_uri, raptor_uri, 1);
 
-  vars_table = rasqal_query_results_get_variables_table(results);
-  rowsource = rasqal_query_results_formatter_get_read_rowsource(world,
-                                                                iostr,
-                                                                formatter,
-                                                                vars_table,
-                                                                base_uri, 0);
-  if(!rowsource)
-    return 1;
+  type = rasqal_query_results_get_type(results);
 
-  while(1) {
-    rasqal_row* row = rasqal_rowsource_read_row(rowsource);
-    if(!row)
-      break;
-    rasqal_query_results_add_row(results, row);
+  /* Read bindings results */
+  if(type == RASQAL_QUERY_RESULTS_BINDINGS) {
+    rasqal_rowsource* rowsource = NULL;
+    rasqal_variables_table* vars_table;
+
+    vars_table = rasqal_query_results_get_variables_table(results);
+    rowsource = rasqal_query_results_formatter_get_read_rowsource(world,
+                                                                  iostr,
+                                                                  formatter,
+                                                                  vars_table,
+                                                                  base_uri, 0);
+    if(!rowsource)
+      return 1;
+
+    while(1) {
+      rasqal_row* row = rasqal_rowsource_read_row(rowsource);
+      if(!row)
+        break;
+      rasqal_query_results_add_row(results, row);
+    }
+
+    if(rowsource)
+      rasqal_free_rowsource(rowsource);
+  } else {
+    rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR, NULL,
+                            "Cannot read '%s' query results format\n",
+                            rasqal_query_results_type_label(type));
+    rc = 1;
   }
 
-  if(rowsource)
-    rasqal_free_rowsource(rowsource);
-  
-  return 0;
+  return rc;
 }
 
 
@@ -584,12 +597,12 @@ rasqal_world_guess_query_results_format_name(rasqal_world* world,
     return NULL;
   
   if(identifier) {
-    unsigned char *p = (unsigned char*)strrchr((const char*)identifier, '.');
+    unsigned char *p = RASQAL_GOOD_CAST(unsigned char*, strrchr(RASQAL_GOOD_CAST(const char*, identifier), '.'));
     if(p) {
       unsigned char *from, *to;
 
       p++;
-      suffix = RASQAL_MALLOC(unsigned char*, strlen((const char*)p) + 1);
+      suffix = RASQAL_MALLOC(unsigned char*, strlen(RASQAL_GOOD_CAST(const char*, p)) + 1);
       if(!suffix)
         return NULL;
 
@@ -634,7 +647,7 @@ rasqal_world_guess_query_results_format_name(rasqal_world* world,
     
     if(uri && factory->desc.uri_strings) {
       int j;
-      const char* uri_string = (const char*)raptor_uri_as_string(uri);
+      const char* uri_string = RASQAL_GOOD_CAST(const char*, raptor_uri_as_string(uri));
       const char* factory_uri_string = NULL;
       
       for(j = 0;
@@ -655,7 +668,8 @@ rasqal_world_guess_query_results_format_name(rasqal_world* world,
 #define FIRSTN 1024
       if(buffer && len && len > FIRSTN) {
         c = buffer[FIRSTN];
-        ((char*)buffer)[FIRSTN] = '\0';
+        /* FIXME - casts a read only const buffer to writeable */
+        (RASQAL_BAD_CAST(char*, buffer))[FIRSTN] = '\0';
       }
 
       score += factory->recognise_syntax(factory, buffer, len, 
@@ -663,12 +677,13 @@ rasqal_world_guess_query_results_format_name(rasqal_world* world,
                                          mime_type);
 
       if(c >= 0)
-        ((char*)buffer)[FIRSTN] = c;
+        /* FIXME - casts a read only const buffer to writeable */
+        (RASQAL_BAD_CAST(char*, buffer))[FIRSTN] = c;
     }
 
     scores[i].score = score < 10 ? score : 10; 
     scores[i].factory = factory;
-#if RASQAL_DEBUG > 2
+#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 2
     RASQAL_DEBUG3("Score %15s : %d\n", factory->desc.names[0], score);
 #endif
   }
