@@ -186,7 +186,7 @@ rasqal_row_print(rasqal_row* row, FILE* fh)
   rasqal_rowsource* rowsource = row->rowsource;
   int i;
   
-  fputs("result[", fh);
+  fputs("row[", fh);
   for(i = 0; i < row->size; i++) {
     /* Do not use rasqal_query_results_get_binding_name(row->results, i); 
      * as it does not work for a construct result
@@ -207,10 +207,7 @@ rasqal_row_print(rasqal_row* row, FILE* fh)
     if(name)
       fprintf(fh, "%s=", name);
 
-    if(value)
-      rasqal_literal_print(value, fh);
-    else
-      fputs("NULL", fh);
+    rasqal_literal_print(value, fh);
   }
 
   if(row->order_size > 0) {
@@ -221,10 +218,7 @@ rasqal_row_print(rasqal_row* row, FILE* fh)
       
       if(i > 0)
         fputs(", ", fh);
-      if(value)
-        rasqal_literal_print(value, fh);
-      else
-        fputs("NULL", fh);
+      rasqal_literal_print(value, fh);
     }
     fputs("]", fh);
 
@@ -239,6 +233,78 @@ rasqal_row_print(rasqal_row* row, FILE* fh)
 }
 
 
+/*
+ * rasqal_row_write:
+ * @row: query result row
+ * @iostr: raptor iostream
+ *
+ * INTERNAL - Write a query result row to an iostream.
+ *
+ * Return value: non-0 on failure
+ */
+int
+rasqal_row_write(rasqal_row* row, raptor_iostream* iostr)
+{
+  rasqal_rowsource* rowsource;
+  int i;
+
+  if(!row || !iostr)
+    return 1;
+
+  rowsource = row->rowsource;
+  raptor_iostream_counted_string_write("row[", 4, iostr);
+  for(i = 0; i < row->size; i++) {
+    /* Do not use rasqal_query_results_get_binding_name(row->results, i);
+     * as it does not work for a construct result
+     */
+    const unsigned char *name = NULL;
+    rasqal_literal *value;
+
+    if(rowsource) {
+      rasqal_variable* v;
+      v = rasqal_rowsource_get_variable_by_offset(rowsource, i);
+      if(v)
+        name = v->name;
+    }
+
+    value = row->values[i];
+    if(i > 0)
+      raptor_iostream_counted_string_write(", ", 2, iostr);
+    if(name) {
+      raptor_iostream_string_write(name, iostr);
+      raptor_iostream_counted_string_write("=", 1, iostr);
+    }
+
+    rasqal_literal_write(value, iostr);
+  }
+
+  if(row->order_size > 0) {
+    raptor_iostream_counted_string_write(" with ordering values [", 23, iostr);
+
+    for(i = 0; i < row->order_size; i++) {
+      rasqal_literal *value = row->order_values[i];
+
+      if(i > 0)
+        raptor_iostream_counted_string_write(", ", 2, iostr);
+      rasqal_literal_write(value, iostr);
+    }
+    raptor_iostream_counted_string_write("]", 1, iostr);
+
+  }
+
+  if(row->group_id >= 0) {
+    raptor_iostream_counted_string_write(" group ", 7, iostr);
+    raptor_iostream_decimal_write(row->group_id, iostr);
+  }
+
+  raptor_iostream_counted_string_write(" offset ", 8, iostr);
+  raptor_iostream_decimal_write(row->offset, iostr);
+  raptor_iostream_counted_string_write("]", 1, iostr);
+
+  return 0;
+}
+
+
 /**
  * rasqal_row_set_value_at:
  * @row: query result row
@@ -344,8 +410,8 @@ rasqal_new_row_sequence(rasqal_world* world,
     }
 
     if(vars_seq) {
-      v = rasqal_new_variable_from_variable(v);
       raptor_sequence_push(vars_seq, v);
+      /* v is now owned by vars_seq */
     }
   }
 
@@ -379,7 +445,7 @@ rasqal_new_row_sequence(rasqal_world* world,
         char *eptr = NULL;
         long number;
         
-        number = strtol((const char*)str, &eptr, 10);
+        number = strtol(RASQAL_GOOD_CAST(const char*, str), &eptr, 10);
         if(!*eptr) {
           /* is an integer */
           l = rasqal_new_numeric_literal_from_long(world,
@@ -399,7 +465,7 @@ rasqal_new_row_sequence(rasqal_world* world,
         /* URI */
         const unsigned char* str;
         raptor_uri* u;
-        str = (const unsigned char*)GET_CELL(row_i, column_i, 1);
+        str = RASQAL_GOOD_CAST(const unsigned char*, GET_CELL(row_i, column_i, 1));
         u = raptor_new_uri(world->raptor_world_ptr, str);
         if(u)
           l = rasqal_new_uri_literal(world, u);
